﻿using System;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Collections.Generic;

namespace PlanCachePolution
{
    public class Program
    {
        public static void Main(string[] args)
        {
            // Console.WriteLine("Calling PlanCachePollution()...");
            // PlanCachePollution();

            // Console.WriteLine("Press ENTER to continue with NoPlanCachePollution()");
            // Console.ReadLine();

            Console.WriteLine("Calling NoPlanCachePollution()...");
            NoPlanCachePollution();

            Console.WriteLine("Done");
            Console.ReadLine();
        }

        private static void PlanCachePollution()
        {
            SqlConnection cnn = new SqlConnection(ConfigurationManager.ConnectionStrings["AdventureWorks"].ConnectionString);

            try
            {
                cnn.Open();

                // Clear the Procedure Cache
                SqlCommand cmd = new SqlCommand("DBCC FreeProcCache", cnn);
                cmd.ExecuteNonQuery();

                DateTime now = DateTime.Now;

                for (int i = 1; i <= 10000; i++)
                {
                    cmd = new SqlCommand(
                    "SELECT * FROM Sales.SalesOrderDetail WHERE SalesOrderDetailID = " + i.ToString(), cnn);
                    SqlDataReader reader = cmd.ExecuteReader();
                    reader.Close();
                }

                TimeSpan elapsedTime = DateTime.Now - now;
                Console.WriteLine("Elapsed time: " + elapsedTime);

                // Retrieve the size of the Plan Cache
                cmd = new SqlCommand("SELECT COUNT(size_in_bytes) AS 'Number of Execution Plans', SUM(size_in_bytes) / 1024 AS 'KB' FROM sys.dm_exec_cached_plans", cnn);
                SqlDataReader reader1 = cmd.ExecuteReader();

                if (reader1.Read())
                {
                    Console.WriteLine("Number of Execution Plans in cache: " + reader1[0]);
                    Console.WriteLine("Plan Cache size: " + reader1[1] + " kb");
                    Console.WriteLine("");
                }
            }
            finally
            {
                cnn.Close();
            }
        }

        private static void NoPlanCachePollution()
        {
            SqlConnection cnn = new SqlConnection(ConfigurationManager.ConnectionStrings["AdventureWorks"].ConnectionString);

            try
            {
                cnn.Open();

                // Clear the Procedure Cache
                SqlCommand cmd = new SqlCommand("DBCC FreeProcCache", cnn);
                cmd.ExecuteNonQuery();

                DateTime now = DateTime.Now;

                for (int i = 1; i <= 10000; i++)
                {
                    cmd = new SqlCommand("SELECT * FROM Sales.SalesOrderDetail WHERE SalesOrderDetailID = @SalesOrderDetailID", cnn);
                    cmd.Parameters.Add(new SqlParameter("@SalesOrderDetailID", SqlDbType.Int));
                    cmd.Parameters["@SalesOrderDetailID"].Value = i;

                    SqlDataReader reader = cmd.ExecuteReader();
                    reader.Close();
                }

                TimeSpan elapsedTime = DateTime.Now - now;
                Console.WriteLine("Elapsed time: " + elapsedTime);

                // Retrieve the size of the Plan Cache
                cmd = new SqlCommand("SELECT COUNT(size_in_bytes) AS 'Number of Execution Plans', SUM(size_in_bytes) / 1024 AS 'KB' FROM sys.dm_exec_cached_plans", cnn);
                SqlDataReader reader1 = cmd.ExecuteReader();

                if (reader1.Read())
                {
                    Console.WriteLine("Number of Execution Plans in cache: " + reader1[0]);
                    Console.WriteLine("Plan Cache size: " + reader1[1] + " kb");
                    Console.WriteLine("");
                }
            }
            finally
            {
                cnn.Close();
            }
        }
    }
}